#!/usr/bin/env python
#*******************************************************************************#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#*******************************************************************************#

import socket, os, threading, time, select 


class Server():
	def __init__(self):
		self.socket_server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.socket_server.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
		self.socket_server.bind(("", 8000))
		self.socket_server.listen(10)
		self.clients = {"name":[], "socket":[]} 
		self.Version = "1.3.0"
		self.thread_lister = threading.Thread(target=self.listen_clients ,args=())
		self.thread_manager = threading.Thread(target=self.manage_clients ,args=())
		
		self.thread_lister.start()
		self.thread_manager.start()
		

	def listen_clients(self):
			while True:
				time.sleep(0.1)
				clientSocket, clientData = self.socket_server.accept()
				nick = clientSocket.recv(512)
				
				if len(self.clients["name"]) < 1:
					clientSocket.send("Eres el primero/a en conectarse!\n")
				else:
					clientSocket.send("Conectado\n")

				self.add_client(clientSocket, nick)
				newClient = str(clientData[0]) + ":" + str(clientData[1])
				self.send_to_clients("Se ha conectado " +  nick, clientSocket)
				print "Se ha conectado " + nick + "(" + newClient + ")"


	def manage_clients(self):
		while True:
			time.sleep(0.1)
			msg=""
			
			for client in self.clients["socket"]:
				try:
					msg = client.recv(2048)
				except:
					print "Error recibiendo datos"
					self.close_client(client)
				
				if msg == "\n":
					msg = ""
						
				if msg == "LSPEERS":
					print "Peticion de lista de conexiones"
					Peers = "LSPEERS="
					for Names in self.clients["name"]:
						Peers += Names + ","
					try:
						client.send(Peers)
						break
					except:
						print "Error enviando peticion"
						pass 
					msg = ""
				
				# La instruccion try esta porque en las pruebas 
				# si el cliente se desconecta a 
				# ultimo momento no se puede volver a reconectar
				#------------------------------------------------
				# instruction  try  are because In testing if 
				# client disconnects last time can not reconnect
				try:
					self.send_to_clients(msg, client)
				except:
					continue


	def add_client(self, skt, nick):
		self.clients["name"].append(nick)
		self.clients["socket"].append(skt)


	def send_to_clients(self, msg, skt):
		if msg == "": 
			return 
		
		skt.send("\nMensaje enviado y recibido")
		now = time.strftime("%Y-%m-%d %H:%M:%S")
		
		for h in self.clients["socket"]:
			if skt != h:
				Name = self.clients["name"][self.clients["socket"].index(skt)]
				message = "\n" + now + " " + Name + " dijo:\n" + msg
				try:
					h.send(message)
				except socket.error:
					print socket.error
					return


	def close_client(self,sock):
		Name = self.clients["name"][self.clients["socket"].index(sock)]	
		try: 
			self.clients["socket"].remove(sock)
			self.clients["name"].remove(Name)	
			sock.close()
			warn="Se ha desconectado " + Name
			self.send_to_clients(warn, sock)
			print warn
		except: 
			print "Error tratando de desconectar"
			return 


	def quit(self):
		print "Saliendo..."
		self.socket_server.shutdown(self.socket_server)
		self.socket_server.close()
		exit(0)


if __name__ == "__main__":
	print "lanchat server"
	lanchat_server = Server()

